@@ -54,16 +54,17 @@ void TvSpielfilmFetcher::fetchGroup(const QString &url,
                 QMap<ChannelId, ChannelData> channels;
                 const QString channelList = matchChannelList.captured(0);
 
-                static const QRegularExpression reChannel(QStringLiteral("<option.*?value=\\\"(.*?)\\\">&nbsp;&nbsp;(.*?)</option>"),
-                                                          QRegularExpression::DotMatchesEverythingOption);
+                static const QRegularExpression reChannel(
+                    QStringLiteral("<option\\s+data-group=\\\"(.*?)\\\"\\s+value=\\\"(.*?)\\\">(&nbsp;&nbsp;)?(.*?)</option>"),
+                    QRegularExpression::DotMatchesEverythingOption);
                 QRegularExpressionMatchIterator it = reChannel.globalMatch(channelList);
                 while (it.hasNext()) {
                     const QRegularExpressionMatch channelMatch = it.next();
-                    const ChannelId id = ChannelId(channelMatch.captured(1));
+                    const ChannelId id = ChannelId(channelMatch.captured(2));
 
                     // exclude groups (e.g. "alle Sender" or "g:1")
-                    if (id.value().length() > 0 && !id.value().contains(QStringLiteral("g:"))) {
-                        const QString name = channelMatch.captured(2);
+                    if (id.value().length() > 0 && !id.value().contains(QStringLiteral("g:")) && !id.value().contains(QStringLiteral("label="))) {
+                        const QString name = channelMatch.captured(4);
                         fetchChannel(id, name, channels);
                     }
                 }
@@ -222,7 +223,10 @@ void TvSpielfilmFetcher::processChannel(const QString &infoTable, const QString
             }
             // it can happen that the stop time is incorrect (i.e. after the next program has already started)
             // overwrite stop time with start time of next program
-            else if (programData.m_startTime < previousProgamData.m_stopTime) {
+            // only if the previous program is really previous:
+            // As days are fetched from future to past to be able to stop early,
+            // the last program of the next day is the "previous" program of the first program of the current day.
+            else if (programData.m_startTime < previousProgamData.m_stopTime && previousProgamData.m_startTime < programData.m_startTime) {
                 previousProgamData.m_stopTime = programData.m_startTime;
             } else {
                 programs.push_back(programData);
@@ -274,11 +278,14 @@ QString TvSpielfilmFetcher::processDescription(const QString &descriptionPage, c
 
     // description
     {
-        static const QRegularExpression re(QStringLiteral("<section class=\\\"broadcast-detail__description\\\">.*?<p>(.*?)</p>"),
+        static const QRegularExpression re(QStringLiteral("<section class=\\\"broadcast-detail__description\\\">.*?(<p>.*?)\\s*</section>"),
                                            QRegularExpression::DotMatchesEverythingOption);
         const QRegularExpressionMatch match = re.match(descriptionPage);
         if (match.hasMatch()) {
-            description += match.captured(1) + QStringLiteral("<br>");
+            QString descriptionHtml = match.captured(1);
+            descriptionHtml.replace(QStringLiteral("<p>"), QStringLiteral(""));
+            descriptionHtml.replace(QStringLiteral("</p>"), QStringLiteral("<br>"));
+            description += descriptionHtml;
         }
     }
 
@@ -290,7 +297,7 @@ QString TvSpielfilmFetcher::processDescription(const QString &descriptionPage, c
             if (!description.isEmpty()) {
                 description += QStringLiteral("<br>");
             }
-            description += i18n("Original title: ") + match.captured(1);
+            description += i18n("Original title: %1", match.captured(1));
         }
     }
 
@@ -302,7 +309,7 @@ QString TvSpielfilmFetcher::processDescription(const QString &descriptionPage, c
             if (!description.isEmpty()) {
                 description += QStringLiteral("<br>");
             }
-            description += i18n("Country: ") + match.captured(1);
+            description += i18n("Country: %1", match.captured(1));
         }
     }
 
@@ -314,7 +321,7 @@ QString TvSpielfilmFetcher::processDescription(const QString &descriptionPage, c
             if (!description.isEmpty()) {
                 description += QStringLiteral("<br>");
             }
-            description += i18n("Year: ") + match.captured(1);
+            description += i18n("Year: %1", match.captured(1));
         }
     }
 
@@ -326,7 +333,7 @@ QString TvSpielfilmFetcher::processDescription(const QString &descriptionPage, c
             if (!description.isEmpty()) {
                 description += QStringLiteral("<br>");
             }
-            description += i18n("Duration: ") + match.captured(1);
+            description += i18n("Duration: %1", match.captured(1));
         }
     }
 
@@ -338,7 +345,7 @@ QString TvSpielfilmFetcher::processDescription(const QString &descriptionPage, c
             if (!description.isEmpty()) {
                 description += QStringLiteral("<br>");
             }
-            description += i18n("FSK: ") + match.captured(1);
+            description += i18n("FSK: %1", match.captured(1));
         }
     }
 
@@ -351,22 +358,31 @@ QString TvSpielfilmFetcher::processDescription(const QString &descriptionPage, c
                 description += QStringLiteral("<br>");
             }
 
-            static const QRegularExpression re(
-                QStringLiteral("<dt.*?>(.*?)</dt>.*?<dd.*?>\\s*(<a\\s*href=\\\".*?\\\"\\s*target=\\\".*?\\\"\\s*title=\\\".*?\\\">)?(.*?)(</a>)?\\s*</dd>"),
-                QRegularExpression::DotMatchesEverythingOption);
+            static const QRegularExpression re(QStringLiteral("<dt.*?>(.*?)</dt>.*?<dd.*?>(.*?)\\s*</dd>"), QRegularExpression::DotMatchesEverythingOption);
             QRegularExpressionMatchIterator it = re.globalMatch(actorsMatch.captured(1));
             while (it.hasNext()) {
                 const QRegularExpressionMatch match = it.next();
                 const QString role = match.captured(1).trimmed();
-                const QString name = match.captured(3).trimmed();
-                if (role == QStringLiteral("&nbsp;") && name == QStringLiteral("&nbsp;")) {
+                static const QRegularExpression reNames(
+                    QStringLiteral("\\s*(<a\\s*href=\\\".*?\\\"\\s*target=\\\".*?\\\"\\s*title=\\\".*?\\\">)?(.+?)(</a>)?\\s*(,|$)"),
+                    QRegularExpression::DotMatchesEverythingOption);
+                QString names = QStringLiteral("");
+                QRegularExpressionMatchIterator namesIt = reNames.globalMatch(match.captured(2));
+                while (namesIt.hasNext()) {
+                    if (!names.isEmpty()) {
+                        names += QStringLiteral(", ");
+                    }
+                    const QRegularExpressionMatch namesMatch = namesIt.next();
+                    names += namesMatch.captured(2).trimmed();
+                }
+                if (role == QStringLiteral("&nbsp;") && names == QStringLiteral("&nbsp;")) {
                     // split cast and crew into 2 sections
                     description += QStringLiteral("<br>");
                 } else {
                     if (role.endsWith(QLatin1Char(':'))) {
-                        description += QStringLiteral("<br>") + role + QLatin1Char(' ') + name;
+                        description += QStringLiteral("<br>") + role + QLatin1Char(' ') + names;
                     } else {
-                        description += QStringLiteral("<br>") + role + QStringLiteral(" - ") + name;
+                        description += QStringLiteral("<br>") + role + QStringLiteral(" - ") + names;
                     }
                 }
             }
